/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.awt.Color;

import org.AlignmentSetting;
import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

/**
 * @author Eva
 */
public class ConstraintPanel extends AbstractStartCalculationAlgorithm {
	
	public String getName() {
		return "<html>" +
							"<font color=\"#04B404\">Parameter setting</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may specify the upper and lower reaction<br>" +
							"bounds:</b><br>" +
							"<br>" +
							"<i>Procedure to alter the default reaction bounds of a given reaction:</i><br>" +
							"<font color=\"#04B404\">STEP 1: </font>Specify the reaction by double clicking on the respective reaction<br>" +
							"node in the graph<br>" +
							"<br>" +
							"<font color=\"#0101DF\">STEP 2:</font> Alter the default value of the lower and upper bounds of the spe-<br>" +
							"cified reaction in the text fields which pops up<br>" +
							"- lower bound: \"below, right\" <br>" +
							"- upper bound: \"above, right\"<br>" +
							"<br>" +
							"Note that the altered  reaction bounds have to correspond to the graphical<br>" +
							"representation of the given reaction. See manual for more detail.<br>" +
							"<br>" +
							"Once the reaction bounds are set, please continue with the analysis by<br>" +
							"chosing the respective analysis technique in the FBA menu.<br>" +
							"<br>";
	}
	
	public void execute() {
		long startO = System.currentTimeMillis();
		try {
			graph.getListenerManager().transactionStarted(this);
			for (GraphElement ge : graph.getGraphElements()) {
				if (ge instanceof Node && FBAHelper.isReaction((Node) ge)) {
					Node n = (Node) ge;
					if (AttributeHelper.isSBMLreversibleReaction(ge)) {
						AttributeHelper.setLabel(1, n, "-1000", "Arial", null);// lb
						AttributeHelper.setLabelAlignment(1, n, AlignmentSetting.BELOWRIGHT);
					} else {
						AttributeHelper.setLabel(1, n, "0", "Arial", null);// lb
						AttributeHelper.setLabelAlignment(1, n, AlignmentSetting.BELOWRIGHT);
					}
					AttributeHelper.setLabel(2, n, "1000", "Arial", null);// ub
					AttributeHelper.setLabelAlignment(2, n, AlignmentSetting.ABOVERIGHT);
					AttributeHelper.setLabelColor(1, n, Color.BLUE);
					AttributeHelper.setLabelColor(2, n, Color.CYAN);
				}
			}
		} catch (Exception err) {
			ErrorMsg.addErrorMessage(err);
		} finally {
			graph.getListenerManager().transactionFinished(this);
		}
		GraphHelper.issueCompleteRedrawForGraph(graph);
		long endO = System.currentTimeMillis();
		MainFrame.showMessage("Parameter setting, processing time " + (endO - startO) + " ms", MessageType.INFO);
	}
	
	@Override
	protected String getImageFileName() {
		return "Parameter.png";
	}
}