/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.TreeSet;

import javax.swing.SwingUtilities;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.BooleanParameter;
import org.graffiti.plugin.parameter.IntegerParameter;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.w3c.dom.Document;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.dbe.algorithms.RemoveMappingDataAlgorithm;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ExperimentInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.Experiment2GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.MapResult;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskStatusProviderSupportingExternalCallImpl;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.ExecutionParameter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBASimVizAddon;
import de.ipk_gatersleben.ag_pbi.fluxbalance.visualization.AnimatedFluxVisualizationSliderMultiFinal2;

/**
 * @author Eva
 */
public class StartCalculationAlgorithmMultiFinalNEW extends AbstractStartCalculationAlgorithm {
	
	private static String ofReaction;
	private static boolean linearOpt = false;
	private static boolean maximization = true;
	// private boolean minimization = false;
	private static String vpReaction1;
	private static int lowLimit1 = 0;
	private static int upperLimit1 = 10;
	private static String vpReaction2;
	private static int lowLimit2 = 0;
	private static int upperLimit2 = 10;
	private static String vpReaction3;
	private static int lowLimit3 = 0;
	private static int upperLimit3 = 10;
	// private final boolean additionalConstraints = false;
	// private boolean SBMLfileGeneration = true;
	private int ofCoef;
	private String nlopt;
	Document doc;
	private int noComputations = 0;
	private int steps1 = 0;
	private int steps2 = 0;
	private int steps3 = 0;
	private double computationTime;
	
	public StartCalculationAlgorithmMultiFinalNEW() {
		super();
	}
	
	public StartCalculationAlgorithmMultiFinalNEW(boolean isTesting) {
		super(isTesting);
	}
	
	public String getName() {
		return "<html><font color=\"#0101DF\">Flux Balance Analysis</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may perform flux balance analysis (FBA)</b><br>" +
							"<br>" +
							"To perform FBA you have to specify:<br>" +
							"(1) the objective function (Reaction to be optimized)<br>" +
							"(2) the mode of optimization (Maximization, Minimization)<br>" +
							"(3) the type of optimization (Linear, Non-linear)<br>" +
							"(4) the input parameters (Reactions to be varied)<br>" +
							"(5) the lower and upper limits of the input parameters (Range <br>" +
							"of parameter variation)<br>" +
							"<br>";
	}
	
	@Override
	public Parameter[] getParameters() {
		
		TreeSet<String> reactionNames = FBAHelper.getReactionNamesUnique(graph.getNodes());
		reactionNames.add(" not specified");
		if (ofReaction == null)
			ofReaction = reactionNames.iterator().next();
		if (vpReaction1 == null)
			vpReaction1 = reactionNames.iterator().next();
		if (vpReaction2 == null)
			vpReaction2 = reactionNames.iterator().next();
		if (vpReaction3 == null)
			vpReaction3 = reactionNames.iterator().next();
		return new Parameter[] {
							new ObjectListParameter(reactionNames.contains(ofReaction) ? ofReaction : reactionNames.iterator().next(), "Reaction to be optimized",
									"Objective function", reactionNames),
							new BooleanParameter(maximization, "Maximization", "Activated checkbox: maximization; not activated checkbox: minimization"),
							// new BooleanParameter(false, "Minimization", "Falls gewählt, wird die OF minimiert...OFcoef = 1"),
				new BooleanParameter(linearOpt, "Linear Optimization",
												"Activated checkbox: linear optimisation; not activated checkbox: nonlinear optimisation (2-step optimisation procedure)"),
							new ObjectListParameter(reactionNames.contains(vpReaction1) ? vpReaction1 : reactionNames.iterator().next(), "Reaction 1 to be varied",
									"input parameter 1", reactionNames),
							new IntegerParameter(lowLimit1, 0, 10, "Lower limit 1", "Minimum value of parameter variation"),
							new IntegerParameter(upperLimit1, 0, 10, "Upper limit 1", "Maximum value of parameter variation"),
							new ObjectListParameter(reactionNames.contains(vpReaction2) ? vpReaction2 : reactionNames.iterator().next(), "Reaction 2 to be varied",
									"input parameter 2", reactionNames),
							new IntegerParameter(lowLimit2, 0, 10, "Lower limit 2", "Minimum value of parameter variation"),
							new IntegerParameter(upperLimit2, 0, 10, "Upper limit 2", "Maximum value of parameter variation"),
							new ObjectListParameter(reactionNames.contains(vpReaction3) ? vpReaction3 : reactionNames.iterator().next(), "Reaction 3 to be varied",
									"input parameter 3", reactionNames),
							new IntegerParameter(lowLimit3, 0, 10, "Lower limit 3", "Minimum value of parameter variation"),
							new IntegerParameter(upperLimit3, 0, 10, "Upper limit 3", "Maximum value of parameter variation"),
		// new BooleanParameter(false, "Account for changed reaction bounds", "Falls gewählt, werden die ge�nderten constraints berücksichtigt..."),
		// new BooleanParameter(false, "Automatic SBML-file generation", "Falls gewählt, wird Berechnung auf neu generiertem SBML-file durchgeführt"),
		
		};
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		
		int i = 0;
		ofReaction = (String) ((ObjectListParameter) params[i++]).getValue();
		maximization = ((BooleanParameter) params[i++]).getBoolean();
		// this.minimization = ((BooleanParameter)params[i++]).getBoolean();
		if (maximization) {
			ofCoef = -1;
		} else {
			ofCoef = 1;
		}
		linearOpt = ((BooleanParameter) params[i++]).getBoolean();
		if (linearOpt) {
			nlopt = "false";
		} else {
			nlopt = "true";
		}
		vpReaction1 = (String) ((ObjectListParameter) params[i++]).getValue();
		lowLimit1 = ((IntegerParameter) params[i++]).getInteger();
		upperLimit1 = ((IntegerParameter) params[i++]).getInteger();
		vpReaction2 = (String) ((ObjectListParameter) params[i++]).getValue();
		lowLimit2 = ((IntegerParameter) params[i++]).getInteger();
		upperLimit2 = ((IntegerParameter) params[i++]).getInteger();
		vpReaction3 = (String) ((ObjectListParameter) params[i++]).getValue();
		lowLimit3 = ((IntegerParameter) params[i++]).getInteger();
		upperLimit3 = ((IntegerParameter) params[i++]).getInteger();
		// this.additionalConstraints = ((BooleanParameter)params[i++]).getBoolean();
		// this.SBMLfileGeneration = ((BooleanParameter)params[i++]).getBoolean();
	}
	
	@Override
	public void check() throws PreconditionException {
		super.check();
		PreconditionException errors = new PreconditionException();
		
		/*
		 * TO DO
		 * if (ofReaction.equals(" not specified")){
		 * errors.add("The reaction to be optimized has to be specified.");
		 * this.ofReaction = null;
		 * }
		 */

		checkForMatlabExecutable(errors, "FBAmultiJan.exe");
		
		HashSet<String> label = new HashSet<String>();
		for (Node n : graph.getNodes()) {
			if (FBAHelper.isReaction(n)) {
				String l = AttributeHelper.getLabel(n, "");
				if (label.contains(l))
					errors.add("Label \"" + l + "\" is used more than once");
				label.add(l);
			}
		}
		/*
		 * if (maximization == true && minimization == true){
		 * errors.add("Please chose one mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 * if (maximization == false && minimization == false){
		 * errors.add("Please specify the mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 */
		if (lowLimit1 < 0) {
			errors.add("The lower limit 1 has to be a positive number.");
			lowLimit1 = 1;
		}
		if (lowLimit2 < 0) {
			errors.add("The lower limit 2 has to be a positive number.");
			lowLimit2 = 1;
		}
		if (lowLimit3 < 0) {
			errors.add("The lower limit 3 has to be a positive number.");
			lowLimit3 = 1;
		}
		if (upperLimit1 < 0) {
			errors.add("The upper limit 1 has to be a positive number.");
			upperLimit1 = 10;
		}
		if (upperLimit2 < 0) {
			errors.add("The upper limit 2 has to be a positive number.");
			upperLimit2 = 10;
		}
		if (upperLimit3 < 0) {
			errors.add("The upper limit 3 has to be a positive number.");
			upperLimit2 = 10;
		}
		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		
		final ExecutionParameter par = FBAHelper.prepareExecution(graph, true);
		if (par == null)
			return;
		
		doExecute(graph, par);
	}
	
	public void doExecute(final Graph graph, final ExecutionParameter par) {
		
		if (ofReaction != null && vpReaction1 != null && ofReaction.equals(vpReaction1)) {
			MainFrame.showMessageDialog("The reaction to be optimized has to be different from the first reaction to be varied.", "Execution Aborted");
			ofReaction = null;
			vpReaction1 = null;
			return;
		}
		if (ofReaction != null && vpReaction2 != null && ofReaction.equals(vpReaction2)) {
			MainFrame.showMessageDialog("The reaction to be optimized has to be different from the second reaction to be varied.", "Execution Aborted");
			ofReaction = null;
			vpReaction2 = null;
			return;
		}
		if (ofReaction != null && vpReaction3 != null && ofReaction.equals(vpReaction3)) {
			MainFrame.showMessageDialog("The reaction to be optimized has to be different from the third reaction to be varied.", "Execution Aborted");
			ofReaction = null;
			vpReaction3 = null;
			return;
		}
		
		final BackgroundTaskStatusProviderSupportingExternalCallImpl status =
				new BackgroundTaskStatusProviderSupportingExternalCallImpl("Zeile 1", "Zeile 2");
		
		FBAHelper.runCalculation(status, getName(), new Runnable() {
			@Override
			public void run() {
				try {
					
					String minVPvalue1;
					String maxVPvalue1;
					String minVPvalue2;
					String maxVPvalue2;
					String minVPvalue3;
					String maxVPvalue3;
					String SBMLmodel = par.filenameSBML;
					String opt = nlopt;
					String OFrct = ofReaction;
					String OFcoef = (new Integer(ofCoef).toString());
					String VPrct1 = vpReaction1;
					
					minVPvalue1 = (new Integer(lowLimit1).toString());
					maxVPvalue1 = (new Integer(upperLimit1).toString());
					if (VPrct1.equals(" not specified")) {
						VPrct1 = "nots*";
					} else {
						steps1 = Integer.parseInt(maxVPvalue1) - Integer.parseInt(minVPvalue1) + 1;
					}
					String VPrct2 = vpReaction2;
					minVPvalue2 = (new Integer(lowLimit2).toString());
					maxVPvalue2 = (new Integer(upperLimit2).toString());
					if (VPrct2.equals(" not specified")) {
						VPrct2 = "nots*";
					} else {
						steps2 = Integer.parseInt(maxVPvalue2) - Integer.parseInt(minVPvalue2) + 1;
					}
					
					String VPrct3 = vpReaction3;
					minVPvalue3 = (new Integer(lowLimit3).toString());;
					maxVPvalue3 = (new Integer(upperLimit3).toString());;
					if (VPrct3.equals(" not specified")) {
						VPrct3 = "nots*";
					} else {
						steps3 = Integer.parseInt(maxVPvalue3) - Integer.parseInt(minVPvalue3) + 1;
					}
					
					// Anzahl Berechnungen ermitteln
					// Voraussetzung: Edukt >0
					int indexZ = 0;
					if (steps1 == 0) {
						steps1 = 1;
						++indexZ;
					}
					if (steps2 == 0) {
						steps2 = 1;
						++indexZ;
					}
					if (steps3 == 0) {
						steps3 = 1;
						++indexZ;
					}
					if (indexZ == 3) {
						noComputations = 0;
					} else {
						noComputations = steps1 * steps2 * steps3;
					}
					
					// computationTime = Math.floor(noComputations * 0.6/60);
					
					// Berechnungsdauer = 4.779*exp 0.0022*x
					computationTime = Math.floor(4.779 * (Math.exp(0.0022 * noComputations)));
					
					if (computationTime == 0) {
						computationTime = 20;
					}
					
					if (computationTime > 40) {
						MainFrame.showMessageDialog(
								"<html>The specified parameter space requires " + noComputations + " computation steps<br>" +
																				"which will take about " + computationTime + " seconds to compute. <br>" +
																				"Reducing the parameter space will speed-up computation.", "Information");
					}
					// exe-file und input/output files muessen im gleichen Ordener sein
					List<String> command = new ArrayList<String>();
					
					String pathExe = FBASimVizAddon.pathExeAddons + "FBAmultiJan.exe";
					
					command.add(pathExe);
					command.add(SBMLmodel);
					command.add(opt);
					command.add(OFrct);
					command.add(OFcoef);
					command.add(VPrct1);
					command.add(VPrct2);
					command.add(VPrct3);
					command.add(minVPvalue1);
					command.add(minVPvalue2);
					command.add(minVPvalue3);
					command.add(maxVPvalue1);
					command.add(maxVPvalue2);
					command.add(maxVPvalue3);
					command.add("NM.txt");
					command.add("VM.txt");
					
					if (par.additionalConstraints == true) {
						command.add("UB.txt");
						command.add("LB.txt");
					} else {
						command.add("null");
						command.add("null");
					}
					
					ProcessBuilder builder = new ProcessBuilder(command);
					builder.directory(
														// working directory: in diesem wird nach den von dem Programm genuzten Files gesucht
							new File(FBASimVizAddon.pathExeAddons));
					
					status.setCurrentStatusText1("Simulation is running");
					status.setCurrentStatusText2("Please wait: the computation will take about " + computationTime + " sec");
					status.setCurrentStatusValue(-1);
					
					final Process godot = builder.start();
					
					godot.getErrorStream().close();
					godot.getInputStream().close();
					godot.getOutputStream().close();
					
					int returnValue = godot.waitFor();
					status.setCurrentStatusValue(100);
					
					if (returnValue > 0) {
						MainFrame.getInstance().showMessageDialog("Possible error. Return Value from simulator: " + returnValue);
					}
					
					// gemappte Daten aus Graph entfernen
					for (GraphElement g : graph.getGraphElements())
						RemoveMappingDataAlgorithm.removeMappingDataFrom(g);
					
					// perform mapping
					Collection<GraphElement> targetGraphElements = new ArrayList<GraphElement>();
					targetGraphElements = graph.getGraphElements();
					
					String filename = par.filenameSBML;
					String filename2 = filename.replaceAll(".xml", "_");
					String filename3 = filename2.replaceAll(".sbml", "_");
					String fullName = par.path + filename3 + "CarbonFluxData.txt";
					String fullNameO = FBASimVizAddon.pathExeAddons + filename3 + "CarbonFluxData.txt";
					
					ExperimentInterface doc = XMLfile.getDocument(fullNameO, status);
					
					if (doc == null) {
						ErrorMsg.addErrorMessage("Experiment was null!");
					} else {
						Experiment2GraphHelper mappingService = new Experiment2GraphHelper();
						MapResult res = mappingService.mapDataToGraphElements(true, doc,
																	targetGraphElements, null, false,
																	GraffitiCharts.HIDDEN.getName(), 0, false, false, false, false, false);
						
						FBAHelper.renameGraphAfterFluxSimulation(graph);
						
						System.out.println("Step1");
						MainFrame.showMessage("Data Mapping completed! (" + res.substanceCount + " fba flux datasets)", MessageType.INFO);
						
						// Resultfile wird in Modellordner verschoben
						
						// FBAHelper.moveFile(fullNameO, fullName);
						FBAHelper.copyFile(fullNameO, fullName);
						
						// xml, VM/NM.txt UB./LB.txt werden gelöscht
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + par.filenameSBML);
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "VM.txt");
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "NM.txt");
						
						System.out.println("Step2");
						if (par.additionalConstraints == true) {
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "UB.txt");
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "LB.txt");
						}
						
						if (!isTesting) {
							// Aufrufen des SliderPanels
							SwingUtilities.invokeLater(new Runnable() {
								public void run() {
									GravistoService.getInstance().runAlgorithm(
																			new AnimatedFluxVisualizationSliderMultiFinal2(), getActionEvent());
								}
							});
						}
						
						// FBAHelper.copyFile(fullNameO, fullName);
						FBAHelper.moveFile(fullNameO, fullName);
						
					}
					
				} catch (Exception e) {
					ErrorMsg.addErrorMessage(e);
				}
				status.setCurrentStatusText2("Processing completed");
				
			}
		});
		// ToDo
		// MainFrame.showMessage("Flux calculation for parameter "+vpReaction1+" and "+vpReaction2+", processing time "+(endO-startO)+" ms", MessageType.INFO);
	}// execute
	
	@Override
	protected String getImageFileName() {
		return "FBA1.png";
	}
	
}