/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import org.BackgroundTaskStatusProviderSupportingExternalCall;
import org.ErrorMsg;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.Experiment;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ExperimentInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.ExperimentDataFileReader;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.GeneExpressionFileReader;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.TableData;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.kegg_expression.KeggExpressionDataset;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.kegg_expression.KeggExpressionReader;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.kegg_expression.TextFileColumnInformation;

public class XMLfile {
	
	public static ExperimentInterface getDocument(String filename, BackgroundTaskStatusProviderSupportingExternalCall status) {
		
		File file = new File(filename);
		
		if (file == null || !file.canRead()) {
			ErrorMsg.addErrorMessage("Internal error: Simulation results from Matlab simulator not written to " + filename + "! Aborting...");
			return null;
		}
		
		TableData expdata = ExperimentDataFileReader.getExcelTableData(file, 0, null, null); // read first 2 rows
		
		KeggExpressionReader ker = new KeggExpressionReader(expdata);
		
		ArrayList<TextFileColumnInformation> relevantColumns = ker.getRawTextFileColumnInformation(false);
		final HashMap<File, ArrayList<TextFileColumnInformation>> rawFile2relevantColumns = new HashMap<File, ArrayList<TextFileColumnInformation>>();
		rawFile2relevantColumns.put(file, relevantColumns);
		ArrayList<KeggExpressionDataset> datasets = new ArrayList<KeggExpressionDataset>();
		
		ArrayList<File> rawTextFiles = new ArrayList<File>();
		rawTextFiles.add(file);
		
		HashMap<File, Boolean> transposeContent = new HashMap<File, Boolean>();
		transposeContent.put(file, false);
		processRawTextFiles(rawTextFiles, datasets, status, rawFile2relevantColumns, transposeContent);
		
		if (datasets.size() > 0) {
			
			KeggExpressionConverterEva kec = new KeggExpressionConverterEva(datasets);
			kec.getDescriptionDataFromUser();
			final TableData resultingData = kec.getDatasetTable();
			if (resultingData != null) {
				ExperimentDataFileReader gefr = new GeneExpressionFileReader();
				gefr.setCoordinator("experiment coordinator");
				gefr.setExperimentName("expression analysis");
				gefr.setTimeUnit("-1");
				gefr.setMeasurementUnit("expression");
				
				try {
					// document = JDOM2DOM.getDOMfromJDOM(gefr.getXMLdata(file, resultingData, null));
					return gefr.getXMLdata(file, resultingData, null);
					
				} catch (Exception err) {
					ErrorMsg.addErrorMessage(err);
					return null;
				}
			}
		}
		return new Experiment();
	}// constructor
	
	private static void processRawTextFiles(ArrayList<File> rawTextFiles,
						ArrayList<KeggExpressionDataset> datasets, BackgroundTaskStatusProviderSupportingExternalCall status,
						HashMap<File, ArrayList<TextFileColumnInformation>> rawTextFile2relevantColumns,
						HashMap<File, Boolean> transposeContent) {
		for (File kef : rawTextFiles) {
			if (rawTextFile2relevantColumns.get(kef) == null)
				continue;
			status.setCurrentStatusText2("Read content of file: " + kef.getName());
			TableData expdata = ExperimentDataFileReader.
									getExcelTableData(kef, -1, rawTextFile2relevantColumns.get(kef), status);
			if (transposeContent.containsKey(kef)) {
				if (transposeContent.get(kef))
					expdata = expdata.getTransposedDataset();
			}
			KeggExpressionReader ker = new KeggExpressionReader(expdata);
			String organism = ker.getOrganism();
			int colGeneId = 1;
			for (TextFileColumnInformation ci : rawTextFile2relevantColumns.get(kef)) {
				if (ci == null)
					continue;
				KeggExpressionDataset ked = new KeggExpressionDataset(ci.getName(), false, "");
				datasets.add(ked);
				int colDataSignal = ci.getSignalColumn();
				Integer colDataSignalQuality = ci.getDetectionColumn();
				status.setCurrentStatusText2("Process File: " + kef.getName() +
									", column(s) " + colDataSignal + "/" + (colDataSignalQuality != null ? colDataSignalQuality : "-"));
				int headerRow = 1;
				for (int row = headerRow + 1; row <= expdata.getMaximumRow(); row++) {
					String geneId;
					String geneIdInInputFile = expdata.getUnicodeStringCellData(colGeneId, row);
					
					if (geneIdInInputFile == null || geneIdInInputFile.length() <= 0) {
						ErrorMsg.addErrorMessage(
										"Internal Error, wrong result file: ID missing in line " + row + " (file " + kef.getName()
												+ "). Ignoring input data in this row!");
						status.setCurrentStatusText2("Internal Error, wrong result file: ID missing in line " + row + ". Check error-log!");
					} else {
						if (organism != null && organism.length() > 0)
							geneId = organism + ":" + geneIdInInputFile;
						else
							geneId = geneIdInInputFile;
						Object val = expdata.getCellData(colDataSignal, row, null);
						if (val != null) {
							try {
								Double cSig;
								if (val instanceof Double)
									cSig = (Double) val;
								else
									if (val instanceof String) {
										String mes_val_s = (String) val;
										if (mes_val_s.equalsIgnoreCase("-inf"))
											cSig = Double.NEGATIVE_INFINITY;
										else {
											if (mes_val_s.equalsIgnoreCase("inf") || mes_val_s.equalsIgnoreCase("+inf"))
												cSig = Double.POSITIVE_INFINITY;
											else {
												if (mes_val_s.equalsIgnoreCase("-") || mes_val_s.equalsIgnoreCase("n/a") || mes_val_s.equalsIgnoreCase("na"))
													cSig = Double.NaN;
												else {
													cSig = null;
													ErrorMsg.addErrorMessage("Invalid dataformat (non-numeric, non 'n/a', 'na', '-') in column " + colDataSignal + ", row "
																		+ row + ", content: " + val);
												}
											}
										}
									} else {
										cSig = null;
										ErrorMsg.addErrorMessage("Invalid dataformat (non-numeric) in column " + colDataSignal + ", row " + row + ", content: " + val);
									}
								if (cSig != null) {
									String signalQuality = null;
									if (colDataSignalQuality != null) {
										signalQuality = expdata.getUnicodeStringCellData(colDataSignalQuality, row);
									}
									ked.addDatapoint(geneId, null, null, null, null, cSig, Double.NaN, Double.NaN, Double.NaN, signalQuality);
								}
							} catch (ClassCastException cce) {
								ErrorMsg.addErrorMessage("Invalid dataformat (non-numeric) in column " + colDataSignal + ", row " + row + ", content: " + val);
							}
						}
					}
				}
			}
		}
		status.setCurrentStatusText2("");
	}
}
