/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.util.Collection;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.NodeHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AppendSBMLAttributes {
	
	public AppendSBMLAttributes(Graph graph) {
		try {
			
			String label;
			
			// alle SBML-Attribute werden neu gesetzt, auch wenn sie schon vorhanden sind
			// alleEdgeLAbel setzen
			
			// alle Knoten durchsuchen
			
			// System.out.println("Step1");
			for (GraphElement ge : graph.getGraphElements()) {
				
				// ModelID
				String modelID = graph.getName();
				String modelID2 = modelID.replaceAll(".gml", "");
				AttributeHelper.setSBMLmodelID(ge, modelID2);
				
				// System.out.println("Step2");
				// Knoten-Bearbeutung
				// Enzyme, Metabolite: SBML Element ID
				if (ge instanceof Node) {
					label = AttributeHelper.getLabel(ge, "");
					AttributeHelper.setSBMLid(ge, label);
					// System.out.println("Step3");
					// Enzyme: Reversibilitaet
					Node n = (Node) ge;
					if (FBAHelper.isReaction(n)) {
						// reversible?
						NodeHelper node = new NodeHelper(n);
						label = AttributeHelper.getLabel(node, "");
						Collection<Edge> inEdges = node.getAllInEdges();
						// Collection<Edge> outEdges = node.getAllOutEdges();
						int sizeIn = inEdges.size();
						// int sizeOut = outEdges.size();
						if (sizeIn > 0) {
							for (Edge inEdge : node.getAllInEdges()) {
								// To Do check, ob alle inEdges gleiche Info rev - irreversible
								String shape = inEdge.getString("graphics.arrowtail");
								String reversible = "org.graffiti.plugins.views.defaults.StandardArrowShape";
								// String irreversible = "";
								if (shape.equals(reversible)) {
									// ausschluß der input rcts
									AttributeHelper.setAttribute(ge, "sbml", "reversible", true);
								} else {
									AttributeHelper.setAttribute(ge, "sbml", "reversible", false);
								}
								break;
							}// for ENzymreversibilität
						} else {
							// inputKnoten
							
							for (Edge outEdge : node.getAllOutEdges()) {
								// To Do check, ob alle inEdges gleiche Info rev - irreversible
								String shape = outEdge.getString("graphics.arrowtail");
								String reversible = "org.graffiti.plugins.views.defaults.StandardArrowShape";
								// String irreversible = "";
								if (shape.equals(reversible)) {
									AttributeHelper.setAttribute(ge, "sbml", "reversible", true);
								} else {
									AttributeHelper.setAttribute(ge, "sbml", "reversible", false);
								}
							}
						}
					}// if Enzym
				}// Enzyme Metabolite: SBML Element ID
				
				// Kanten: Role, Reversibility
				if (ge instanceof Edge) {
					
					// edgeLAbel Setzen
					String EdgeLabel = AttributeHelper.getLabel(ge, "noLabel");
					// System.out.println("EdgeLabel: "+EdgeLabel);
					if (EdgeLabel.equals("noLabel")) {
						// System.out.println("!!!!!!!!!!!!!!!!!!");
						AttributeHelper.setLabel(ge, "");
					}
					
					// System.out.println("Step4");
					// Reversibility
					Edge e = (Edge) ge;
					String shape = e.getString("graphics.arrowtail");
					String reversible = "org.graffiti.plugins.views.defaults.StandardArrowShape";
					if (shape.equals(reversible)) {
						AttributeHelper.setAttribute(ge, "sbml", "reversible", true);
					} else {
						AttributeHelper.setAttribute(ge, "sbml", "reversible", false);
					}
					
					// Role
					Node nT = e.getTarget();
					if (nT != null) {
						if (FBAHelper.isReaction(nT)) {
							AttributeHelper.setSBMLrole(e, "reactant");
							label = AttributeHelper.getLabel(nT, "");
							AttributeHelper.setSBMLid(e, label);
						} else {
							AttributeHelper.setSBMLrole(e, "product");
							Node nS = e.getSource();
							label = AttributeHelper.getLabel(nS, "");
							AttributeHelper.setSBMLid(e, label);
						}
					}
				}
			}// for graph elements
			// System.out.println("Step5");
		} catch (final Exception err) {
			ErrorMsg.addErrorMessage(err);
		}
	}
}
