package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Graph;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.selection.Selection;

import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;

public class CompleteReconstruction extends AbstractStartCalculationAlgorithm {
	
	private final AbstractStartCalculationAlgorithm sbml;
	private final CreateMetaboliteFile metab;
	private final CreateCmatrix cmat;
	private final AbstractStartCalculationAlgorithm reac;
	
	public CompleteReconstruction() {
		super();
		sbml = new CreateSBMLFile();
		metab = new CreateMetaboliteFile();
		cmat = new CreateCmatrix();
		reac = new CreateReactionFile();
	}
	
	@Override
	public void check() throws PreconditionException {
		super.check();
		sbml.check();
		metab.check();
		cmat.check();
		reac.check();
	}
	
	@Override
	public void attach(Graph graph, Selection selection) {
		super.attach(graph, selection);
		sbml.attach(graph, selection);
		metab.attach(graph, selection);
		cmat.attach(graph, selection);
		reac.attach(graph, selection);
	}
	
	@Override
	protected String getImageFileName() {
		return null;
	}
	
	@Override
	public void execute() {
		SBMLfile s = new SBMLfile(graph);
		if (!s.erroroccured) {
			HashMap<String, Integer> metabolites = CreateMetaboliteFile.getMetabolitesByDialog(graph.getNodes());
			CreateMetaboliteFile.resetAllSBMLAttributes(graph);
			int index = CreateMetaboliteFile.createMetaboliteFile(CreateMetaboliteFile.getMetaboliteFilenameFromGraph(graph), metabolites);
			if (index != -1) {
				ArrayList<String> unblncreact = CreateCmatrix.doCreateMatrix(new File(CreateMetaboliteFile.getMetaboliteFilenameFromGraph(graph)), graph);
				if (unblncreact != null) {
					Reactionfile rf = new Reactionfile(graph);
					String unbalancedText = unblncreact.size() > 0 ?
							", but the following reactions<br>are unbalanced:<br>" + CreateCmatrix.getUnbalancedWarningList(unblncreact) : ",";
					if (rf.erroroccured)
						MainFrame.showMessageDialog(
								"<html>All files, excluding the optional reaction file, successfully created" + unbalancedText, "Information");
					else
						MainFrame.showMessageDialog("<html>All files successfully created" + unbalancedText, "Information");
				}
			}
		}
		
	}
	
	@Override
	public String getName() {
		return "<html><font color=\"#04B4AE\">Create All</font>";
	}
	
	@Override
	public boolean mayWorkOnMultipleGraphs() {
		return true;
	}
	
}
