/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.AttributeHelper;
import org.ErrorMsg;
import org.OpenFileDialogService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.PreconditionException;

import de.ipk_gatersleben.ag_nw.graffiti.MyInputHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.databases.kegg.CompoundService;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.NodeHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;

/**
 * @author Eva
 */
public class CreateMetaboliteFile extends AbstractStartCalculationAlgorithm {
	
	private boolean fillFileByDialog = true;
	
	public CreateMetaboliteFile() {
		this(true);
	}
	
	public CreateMetaboliteFile(boolean fillFileByDialog) {
		this.fillFileByDialog = fillFileByDialog;
	}
	
	public String getName() {
		return "<html>" +
							"<font color=\"#04B404\">Create Metabolite file</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may create a metabolite file which<br>" +
							"is required for subsequent C-matrix generation.</b><br>" +
							"<br>" +
							"The Metabolite file which lists all of the metabolites of a given <br>" +
							"metabolic network map as well as their respective number of C-atoms <br>" +
							"(See Example), is required for subsequent C-matrix generation.<br>" +
							"<br>" +
							"In case the given metabolic model contains metabolites not available <br>" +
							"in the KEGG database, the number of C-atoms for these metabolites<br>" +
							"is missing in the generated Metabolite file and the file hast to be ma-<br>" +
							"nually completed by adding the number of C-atoms behind each of<br>" +
							"these metabolites as shown in the Example.<br>";
	}
	
	@Override
	public void check() throws PreconditionException {
		
		super.check();
		PreconditionException errors = new PreconditionException();
		
		checkForCorrectShapes(errors, graph);
		
		checkForUnconnectedNodes(errors, graph);
		
		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		try {
			// create MetabiliteFile-Name
			String absPathfileName = getMetaboliteFilenameFromGraph(graph);
			
			resetAllSBMLAttributes(graph);
			// for graph elements
			// HENDRIK: habe das hier mal optimiert, aber die alten methoden sind noch da (nur wegkommentiert)
			// // Metabolite-File erzeugen
			// HashMap<String, ArrayList<Comparable<?>>> listOfMetabolites = new HashMap<String, ArrayList<Comparable<?>>>();
			// // Übergabe der Metabolite-SBML-ids & # C-atome
			// listOfMetabolites = getMetabolites(graph.getNodes());
			//
			// ArrayList<Comparable<?>> metabolitesNames = new ArrayList<Comparable<?>>();
			// metabolitesNames = listOfMetabolites.get("names");
			//
			// ArrayList<Comparable<?>> metabolitesCatoms = new ArrayList<Comparable<?>>();
			// metabolitesCatoms = listOfMetabolites.get("atoms");
			//
			// // Neu TreeSet mit "fertigen" Strings basteln
			// TreeSet<String> metaboliteString = new TreeSet<String>();
			// Iterator<Comparable<?>> iterN = metabolitesNames.iterator();
			// Iterator<Comparable<?>> iterA = metabolitesCatoms.iterator();
			//
			// int index = 0;
			//
			// while (iterN.hasNext()) {
			// String metabol = (String) iterN.next();
			// int atoms = (Integer) iterA.next();
			// String metaboliteSTRING = "";
			// if (atoms == 0) {
			// metaboliteSTRING = metabol + ": ";
			// // MetaboliteFile.write(metabol+": ");
			// index++;
			// } else {
			// metaboliteSTRING = metabol + ": " + Integer.toString(atoms);
			// }
			// metaboliteString.add(metaboliteSTRING);
			//
			// }
			// createMetaboliteFile2(absPathfileName, metaboliteString);
			
			// createFile
			int index = -1;
			if (fillFileByDialog)
				index = createMetaboliteFile(absPathfileName, getMetabolitesByDialog(graph.getNodes()));
			else
				index = createMetaboliteFile(absPathfileName, getMetabolites(graph.getNodes()));
			
			switch (index) {
				case -1:
					break;
				case 0:
					MainFrame.showMessage("Metabolite file created", MessageType.INFO);
					break;
				default:
					MainFrame.showMessageDialog(
							"<html>" +
									"Metabolite file created. There are metabolites with missing<br>" +
									"C-atom values, please manually fill in missing data (default 1).<br>" +
									"See manual, for detailed information.",
							"Information");
					break;
			}
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
		
	}// execute
	
	public static void resetAllSBMLAttributes(Graph graph) {
		String label;
		// alle SBML-Attribute werden neu gesetzt, auch wenn sie schon vorhanden sind
		for (GraphElement ge : graph.getGraphElements()) {
			if (ge instanceof Node) {
				label = AttributeHelper.getLabel(ge, "");
				AttributeHelper.setSBMLid(ge, label);
				// System.out.println(label);
			}
		}
	}
	
	public static String getMetaboliteFilenameFromGraph(Graph graph) {
		return graph.getName(true).replaceAll(".gml", "_Metabolite.txt");
	}
	
	public static int createMetaboliteFile(String absPathfileName2, HashMap<String, Integer> metabolites) {
		if (metabolites == null)
			return -1;
		int index = 0;
		try {
			BufferedWriter MetaboliteFile = new BufferedWriter(new FileWriter(absPathfileName2));
			
			for (String metabol : metabolites.keySet()) {
				int catoms = metabolites.get(metabol);
				if (catoms == 0) {
					MetaboliteFile.write(metabol + ": 1");
					index++;
				} else
					MetaboliteFile.write(metabol + ": " + catoms);
				
				MetaboliteFile.newLine();
			}
			MetaboliteFile.close();
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
			return -1;
		}
		return index;
	}
	
	// public void createMetaboliteFile2(String absPathfileName2, TreeSet<String> metaboliteString) {
	//
	// try {
	// BufferedWriter MetaboliteFile = new BufferedWriter(new FileWriter(absPathfileName2));
	// Iterator<String> iterN = metaboliteString.iterator();
	//
	// while (iterN.hasNext()) {
	// String metabol = iterN.next();
	// MetaboliteFile.write(metabol + "1");
	// MetaboliteFile.newLine();
	// }
	// MetaboliteFile.close();
	// } catch (IOException e1) {
	// ErrorMsg.addErrorMessage(e1);
	// }
	//
	// }
	
	public static HashMap<String, Integer> getMetabolites(Collection<Node> nodes) {
		
		HashMap<String, Integer> metabolites = new HashMap<String, Integer>();
		
		for (Node n : nodes) {
			NodeHelper node = new NodeHelper(n);
			String label = AttributeHelper.getLabel(n, "");
			
			if (label != null) {
				if (AttributeHelper.getShape(n).contains("circle")) {
					
					String metabolite = org.AttributeHelper.getSBMLid(node);
					if (metabolites.containsKey(metabolite))
						continue;
					
					metabolites.put(metabolite, CompoundService.getNumberOfCarbonAtoms(metabolite));
				}
			}
		}
		return metabolites;
	}
	
	// public static HashMap<String, ArrayList<Comparable<?>>> getMetabolites(Collection<Node> nodes) {
	//
	// HashMap<String, ArrayList<Comparable<?>>> metabolites = new HashMap<String, ArrayList<Comparable<?>>>();
	// ArrayList<Comparable<?>> metabolitesNames = new ArrayList<Comparable<?>>();
	// ArrayList<Comparable<?>> metabolitesCatoms = new ArrayList<Comparable<?>>();
	// for (Node n : nodes) {
	// NodeHelper node = new NodeHelper(n);
	// String label = AttributeHelper.getLabel(n, "");
	//
	// if (label != null) {
	// if (AttributeHelper.getShape(n).contains("circle")) {
	//
	// String metabolite = org.AttributeHelper.getSBMLid(node);
	// metabolitesNames.add(metabolite);
	// int Ccount = CompoundService.getNumberOfCarbonAtoms(metabolite);
	// metabolitesCatoms.add(Ccount);
	// // System.out.println("metabolite: "+sbmlID);
	// metabolites.put("names", metabolitesNames);
	// metabolites.put("atoms", metabolitesCatoms);
	// }
	// }
	// }
	// return metabolites;
	// }
	
	@Override
	protected String getImageFileName() {
		return "MetaboliteFile_15.png";
	}
	
	public static File askUserForMetaboliteFile(File activeDir) {
		// Einlesen des metabolite-files (*.xml)
		if (activeDir != null)
			OpenFileDialogService.setActiveDirectoryFrom(activeDir);
		File metabf = OpenFileDialogService.getFile(
				new String[] { ".txt" }, "Metabolite File (*.txt)", "Select Metabolite File");
		
		if (metabf == null) {
			MainFrame.showMessageDialog("Please select a Metabolite File (*.txt)!", "Error");
			return null;
		} else
			return metabf;
	}
	
	public static HashMap<String, Integer> getMetabolitesByDialog(List<Node> nodes) {
		return getMetabolitesByDialog(nodes, true);
	}
	
	public static HashMap<String, Integer> getMetabolitesByDialog(List<Node> nodes, boolean useExistingFile) {
		HashMap<String, Integer> metabolitesFromFile = null;
		HashMap<String, Integer> metabolitesFromGraph = getMetabolites(nodes);
		if (useExistingFile) {
			// get all values for the metabolites existing in the actual graph (discards metabolite values of deleted metabolites)
			metabolitesFromFile = readMetaboliteFile(nodes.iterator().next().getGraph());
			if (metabolitesFromFile != null) {
				for (Entry<String, Integer> e : metabolitesFromFile.entrySet())
					if (metabolitesFromGraph.containsKey(e.getKey()))
						metabolitesFromGraph.put(e.getKey(), e.getValue());
			}
		} else
			return metabolitesFromGraph;
		
		return getMetabolitesByDialog(metabolitesFromGraph);
		
	}
	
	public static HashMap<String, Integer> readMetaboliteFile(Graph graph) {
		if (graph == null)
			return null;
		
		File metabf = new File(getMetaboliteFilenameFromGraph(graph));
		if (!metabf.canRead())
			return null;
		
		HashMap<String, Integer> metabolites = new HashMap<String, Integer>();
		try {
			BufferedReader metaboliteFile = new BufferedReader(new FileReader(getMetaboliteFilenameFromGraph(graph)));
			String line;
			while ((line = metaboliteFile.readLine()) != null) {
				String[] spl = line.split(": ");
				if (spl == null || spl.length != 2)
					continue;
				try {
					Integer.parseInt(spl[1]);
				} catch (NumberFormatException e) {
					continue;
				}
				metabolites.put(spl[0], Integer.parseInt(spl[1]));
			}
			metaboliteFile.close();
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
			return null;
		}
		return metabolites;
	}
	
	public static HashMap<String, Integer> getMetabolitesByDialog(HashMap<String, Integer> metabolites) {
		
		Object[] params = new Object[metabolites.size() * 2];
		int i = 0;
		for (String metabol : metabolites.keySet()) {
			params[i++] = metabol;
			params[i++] = metabolites.get(metabol) == 0 ? 1 : metabolites.get(metabol);
		}
		Object[] res = MyInputHelper.getInput("<html>Please enter the number of C-atoms of<br>" +
																"the substances listed below. These are<br>" +
																"needed for creating the metabolite file.", "Choose Atom Numbers", params);
		if (res == null) {
			MainFrame.showMessageDialog("Reconstruction aborted by user.", "Information");
			return null;
		}
		HashMap<String, Integer> newmetabolites = new HashMap<String, Integer>();
		i = 0;
		for (String metabol : metabolites.keySet()) {
			int atoms = (Integer) res[i++];
			if (atoms < 0) {
				MainFrame.showMessageDialog("No negative numbers of C-atoms allowed. Aborting...", "Error");
				return null;
			}
			newmetabolites.put(metabol, atoms);
		}
		return newmetabolites;
	}
}