/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;

import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class Reactionfile {
	
	String fileName = null;
	boolean erroroccured = false;
	
	Reactionfile(Graph graph) {
		
		new AppendSBMLAttributes(graph);
		
		// Reaction-File erzeugen
		
		// create SBMLfile-Name
		String absPathfileName = getReactionFilenameFromGraph(graph);
		
		// modelID
		String modelID = graph.getName();
		String modelID2 = modelID.replaceAll(".gml", "");
		
		ArrayList<String> listOfReactions = new ArrayList<String>();
		listOfReactions = getReactionList(graph.getNodes());
		
		HashMap<String, ArrayList<String>> resultArrays = getReactionList2(graph.getNodes());
		listOfReactions = resultArrays.get("listSBML");
		
		// create Reactionfile
		createReactionfile(absPathfileName, modelID2, listOfReactions);
		
	}
	
	public static String getReactionFilenameFromGraph(Graph graph) {
		return graph.getName(true).replaceAll(".gml", ".txt");
	}
	
	String getName() {
		return fileName;
	}
	
	public void createReactionfile(String absPathfileName2, String modelID2, ArrayList<String> listOfReactions) {
		try {
			BufferedWriter Reactionfile = new BufferedWriter(new FileWriter(absPathfileName2));
			Reactionfile.write("Model: " + modelID2);
			Reactionfile.newLine();
			Iterator<String> iterR = listOfReactions.iterator();
			while (iterR.hasNext()) {
				String rct = iterR.next();
				Reactionfile.write(rct);
				Reactionfile.newLine();
			}
			Reactionfile.close();
			
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
			erroroccured = true;
		}
		
	}
	
	public void createCmatrixfile(String absPathfileName2, String modelID2, ArrayList<?> listOfReactionsCmatrix, TreeSet<String> listOfMetabolites) {
		
		try {
			
			BufferedWriter Cmatrixfile = new BufferedWriter(new FileWriter(absPathfileName2));
			Cmatrixfile.write("Model:" + modelID2);
			Cmatrixfile.newLine();
			Cmatrixfile.write(":::Matrix 1:::::::::::::::::Matrix 2::::::::::::::control");
			Cmatrixfile.newLine();
			Cmatrixfile
								.write("reaction name:reaction::1.St:1.Ed:2.St:2.Ed:3.St:3.Ed:4.St:4.Ed:1.Pst:1.Prod:2.Pst:2.Prod:3.Pst:3.Prod:4.Pst:4.Prod::1.St:1.Ed:2.St:2.Ed:3.St:3.Ed:4.St:4.Ed:1.Pst:1.Prod:2.Pst:2.Prod:3.Pst:3.Prod:4.Pst:4.Prod:sum:sum:result");
			Cmatrixfile.newLine();
			Iterator<?> iterR = listOfReactionsCmatrix.iterator();
			while (iterR.hasNext()) {
				String rct = (String) iterR.next();
				Cmatrixfile.write(rct);
				Cmatrixfile.newLine();
			}
			Cmatrixfile.close();
			
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
		}
	}
	
	// public static TreeSet<String> getReactionNames(Collection<Node> nodes) {
	//		
	// TreeSet<String> reactions = new TreeSet<String>();
	// for (Node n : nodes) {
	// NodeHelper node = new NodeHelper(n);
	// String label = AttributeHelper.getLabel(n, "");
	//			
	// if (label != null) {
	// if (FBAHelper.isReaction(n)) {
	// String sbmlID = org.AttributeHelper.getSBMLid(node);
	// reactions.add(sbmlID);
	// }
	// }
	// }
	// return reactions;
	// }
	//	
	// public static TreeSet<String> getMetaboliteNames(Collection<Node> nodes) {
	//		
	// TreeSet<String> metabolites = new TreeSet<String>();
	// for (Node n : nodes) {
	// NodeHelper node = new NodeHelper(n);
	// String label = AttributeHelper.getLabel(n, "");
	//			
	// if (label != null) {
	// if (AttributeHelper.getShape(n).contains("circle")) {
	// String sbmlID = org.AttributeHelper.getSBMLid(node);
	// metabolites.add(sbmlID);
	// }
	// }
	// }
	// return metabolites;
	// }
	
	public static String getTab(int d) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < d; i++)
			sb.append("\t");
		return sb.toString();
	}
	
	public static String createRctString(String rctLabel, String rev, ArrayList<String> reactants, ArrayList<String> products) {
		
		String RctString = "";
		String append1 = rctLabel + ": " + getTab(1);
		String append2 = "";
		Iterator<String> iter = reactants.iterator();
		int index = 0;
		while (iter.hasNext()) {
			index++;
			String reactantLabel = iter.next();
			String reactantStoichcoef = iter.next();
			String append3;
			// System.out.println("indexReactant: "+index);
			if (index > 1) {
				append3 = "+ " + reactantStoichcoef + " " + reactantLabel + " ";
			} else {
				append3 = reactantStoichcoef + " " + reactantLabel + " ";
			}
			append2 = append2 + append3;
		}
		
		String reversible = " <==> ";
		String irrev = " ==> ";
		String append4;
		if (rev.equals("true")) {
			append4 = reversible;
		} else {
			append4 = irrev;
		}
		
		String append5 = "";
		int index1 = 0;
		Iterator<String> iterP = products.iterator();
		while (iterP.hasNext()) {
			index1++;
			String productLabel = iterP.next();
			String productStoichcoef = iterP.next();
			String append6;
			if (index1 > 1) {
				append6 = "+ " + productStoichcoef + " " + productLabel + " ";
			} else {
				append6 = productStoichcoef + " " + productLabel + " ";
			}
			append5 = append5 + append6;
		}
		
		RctString = append1 + append2 + append4 + append5;
		return RctString;
	}
	
	public static String createCmatrixString(String rctLabel, String rev, ArrayList<String> reactants, ArrayList<String> products) {
		
		String CmatrixString = "";
		CmatrixString = createRctString(rctLabel, rev, reactants, products);
		
		String append2 = "";
		Iterator<String> iter = reactants.iterator();
		int index = 0;
		while (iter.hasNext()) {
			index++;
			String reactantLabel = iter.next();
			String reactantStoichcoef = iter.next();
			String append3;
			if (index > 1 && index <= 4) {
				append3 = "+ " + reactantStoichcoef + ": " + reactantLabel + ": ";
			} else
				if (index <= 1) {
					append3 = reactantStoichcoef + ": " + reactantLabel + ": ";
				} else {
					break;
				}
			append2 = append2 + append3;
		}
		
		index++;
		String fill = "";
		while (index <= 4) {
			switch (index) {
				case 1:
					fill = "0::0::0::0::";
					break;
				case 2:
					fill = "0::0::0::";
					break;
				case 3:
					fill = "0::0::";
					break;
				case 4:
					fill = "0::";
					break;
				default:
					System.out.println("Error");
			}
			index++;
		}
		String reactant = append2 + fill;
		
		String append5 = "";
		int index1 = 0;
		Iterator<String> iterP = products.iterator();
		while (iterP.hasNext()) {
			index1++;
			String productLabel = iterP.next();
			String productStoichcoef = iterP.next();
			String append6;
			if (index1 > 1 && index1 <= 4) {
				append6 = "+" + productStoichcoef + ": " + productLabel + ": ";
			} else
				if (index <= 1) {
					append6 = productStoichcoef + ": " + productLabel + ": ";
				} else {
					break;
				}
			append5 = append5 + append6;
		}
		index1++;
		String fill1 = "";
		while (index1 <= 4) {
			switch (index1) {
				case 1:
					fill1 = "0::0::0::0::";
					break;
				case 2:
					fill1 = "0::0::0::";
					break;
				case 3:
					fill1 = "0::0::";
					break;
				case 4:
					fill1 = "0::";
					break;
				default:
					System.out.println("Error");
			}
			index1++;
		}
		
		String product = append5 + fill1;
		String matrix2 = ":::::::::::::::::::^:";
		CmatrixString = CmatrixString + ": :" + reactant + product + matrix2;
		
		return CmatrixString;
	}
	
	public static ArrayList<String> getReactionList(Collection<Node> nodes) {
		
		String rev = "";
		ArrayList<String> listOfRcts = new ArrayList<String>();
		for (Node n : nodes) {
			if (FBAHelper.isReaction(n)) {
				
				String rctLabel = AttributeHelper.getLabel(n, "");
				if (AttributeHelper.isSBMLreversibleReaction(n)) {
					rev = "true";
				} else {
					rev = "false";
				}
				ArrayList<String> reactants = new ArrayList<String>();
				ArrayList<String> products = new ArrayList<String>();
				
				for (Edge inEdge : n.getAllInEdges()) {
					
					// System.out.println("Step8_2B");
					Node nS = inEdge.getSource();
					if (nS != null) {
						String reactantLabel = AttributeHelper.getLabel(nS, "");
						reactants.add(reactantLabel);
						String stoichCoefR = AttributeHelper.getLabel(inEdge, "");
						if (stoichCoefR.equals("")) {
							stoichCoefR = "1";
						}
						// Hinter Reactantname wird direkt SToichCoefficient eingelesen
						reactants.add(stoichCoefR);
					}
				}
				
				for (Edge outEdge : n.getAllOutEdges()) {
					Node nT = outEdge.getTarget();
					if (nT != null) {
						String productLabel = AttributeHelper.getLabel(nT, "");
						products.add(productLabel);
						String stoichCoefP = AttributeHelper.getLabel(outEdge, "");
						if (stoichCoefP.equals("")) {
							stoichCoefP = "1";
						}
						products.add(stoichCoefP);
					}
				}
				String SBMLString = createRctString(rctLabel, rev, reactants, products);
				listOfRcts.add(SBMLString);
				
			}
		}
		return listOfRcts;
	}
	
	public static HashMap<String, ArrayList<String>> getReactionList2(Collection<Node> nodes) {
		
		// Cmatrix nicht mehr notwendig
		
		String rev = "";
		HashMap<String, ArrayList<String>> result = new HashMap<String, ArrayList<String>>();
		ArrayList<String> listOfRcts = new ArrayList<String>();
		ArrayList<String> listOfCmatrixRcts = new ArrayList<String>();
		for (Node n : nodes) {
			if (FBAHelper.isReaction(n)) {
				String rctLabel = AttributeHelper.getLabel(n, "");
				if (AttributeHelper.isSBMLreversibleReaction(n)) {
					rev = "true";
				} else {
					rev = "false";
				}
				ArrayList<String> reactants = new ArrayList<String>();
				ArrayList<String> products = new ArrayList<String>();
				
				for (Edge inEdge : n.getAllInEdges()) {
					Node nS = inEdge.getSource();
					if (nS != null) {
						String reactantLabel = AttributeHelper.getLabel(nS, "");
						reactants.add(reactantLabel);
						String stoichCoefR = AttributeHelper.getLabel(inEdge, "");
						if (stoichCoefR.equals("")) {
							stoichCoefR = "1";
						}
						
						// Hinter Reactantname wird direkt SToichCoefficient eingelesen
						reactants.add(stoichCoefR);
					}
				}
				
				for (Edge outEdge : n.getAllOutEdges()) {
					Node nT = outEdge.getTarget();
					if (nT != null) {
						String productLabel = AttributeHelper.getLabel(nT, "");
						products.add(productLabel);
						String stoichCoefP = AttributeHelper.getLabel(outEdge, "");
						if (stoichCoefP.equals("")) {
							stoichCoefP = "1";
						}
						products.add(stoichCoefP);
					}
				}
				
				String SBMLString = createRctString(rctLabel, rev, reactants, products);
				listOfRcts.add(SBMLString);
				
			}
		}
		result.put("listSBML", listOfRcts);
		result.put("listCmatrix", listOfCmatrixRcts);
		return result;
	}
}
