package de.ipk_gatersleben.ag_pbi.fluxbalance.startsim;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.TreeSet;

import javax.swing.JOptionPane;

import org.AttributeHelper;
import org.BackgroundTaskStatusProviderSupportingExternalCall;
import org.ErrorMsg;
import org.StringManipulationTools;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.actions.FileSaveAction;
import org.graffiti.graph.AdjListGraph;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.session.EditorSession;

import de.ipk_gatersleben.ag_nw.graffiti.MyInputHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.NodeHelper;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.Cmatrix;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateCmatrix;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateSBMLFile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.ModelReconstructionSelection;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.SBMLfile;

public class FBAHelper {
	
	private static final String GRAPH_SIMULATED = "_fluxsimulation";
	
	public static boolean isReaction(Node n) {
		String shape = AttributeHelper.getShape(n);
		// Vanted may have nodes without a shape-attribute, which will be visualised as rectangles
		return shape == null || shape.length() <= 0 || shape.contains("Rectangle") || shape.contains("rectangle");
	}
	
	public static boolean checkConstraints(Graph graph) {
		
		boolean res = true;
		for (GraphElement ge : graph.getGraphElements()) {
			if (ge instanceof Node && FBAHelper.isReaction((Node) ge)) {
				Node n = (Node) ge;
				String lb = AttributeHelper.getLabel(1, n, "");
				String ub = AttributeHelper.getLabel(2, n, "");
				if (lb.equals("") && ub.equals("")) {
					res = false;
					break;
				}
			}
		}
		return res;
		
	}
	
	public static ArrayList<String> getReactionNames(Collection<Node> nodes) {
		
		ArrayList<String> reactions = new ArrayList<String>();
		for (Node n : nodes) {
			String label = AttributeHelper.getLabel(n, null);
			
			if (label != null) {
				if (FBAHelper.isReaction(n)) {
					// String sbmlID = org.AttributeHelper.getSBMLid(node);
					reactions.add(label);
				}
			}
		}
		return reactions;
	}
	
	public static TreeSet<String> getReactionNamesUnique(Collection<Node> nodes) {
		return new TreeSet<String>(getReactionNames(nodes));
	}
	
	public static TreeSet<String> getMetaboliteNamesUnique(Collection<Node> nodes) {
		return new TreeSet<String>(getMetaboliteNames(nodes));
	}
	
	// Graph-check
	public static ArrayList<String> getMetaboliteNames(Collection<Node> nodes) {
		
		ArrayList<String> metabolites = new ArrayList<String>();
		for (Node n : nodes) {
			String label = AttributeHelper.getLabel(n, null);
			
			if (label != null) {
				if (AttributeHelper.getShape(n).contains("circle")) {
					// NEU: Uebergabe der Knoten-Label nicht SBML-IDs
					// String sbmlID = org.AttributeHelper.getSBMLid(node);
					// System.out.println("metabolite: "+sbmlID);
					metabolites.add(label);
				}
			}
		}
		return metabolites;
	}
	
	public static int getNumberOfNodes(Collection<Node> nodes) {
		int nodesNo = nodes.size();
		return nodesNo;
	}
	
	public static ArrayList<String> getSingleNodes(Collection<Node> nodes) {
		
		ArrayList<String> singleNodes = new ArrayList<String>();
		
		for (Node n : nodes) {
			NodeHelper node = new NodeHelper(n);
			String label = AttributeHelper.getLabel(n, null);
			
			if (label != null) {
				Collection<Edge> inEdges = node.getAllInEdges();
				int inE = inEdges.size();
				Collection<Edge> outEdges = node.getAllOutEdges();
				int outE = outEdges.size();
				if ((inE + outE) == 0) {
					// String sbmlID = org.AttributeHelper.getSBMLid(node);
					singleNodes.add(label);
					node.setLabel("Delete");
				}
			}
		}
		return singleNodes;
	}
	
	public static TreeSet<String> getSingleNodesUnique(Collection<Node> nodes) {
		return new TreeSet<String>(getSingleNodes(nodes));
	}
	
	public static void deleteFile(String file) {
		
		File f1 = new File(file);
		boolean success = f1.delete();
		if (!success)
			System.err.println("File deletion failed. " + file);
	}
	
	public static void moveFile(String src, String dest) {
		
		File quelle = new File(src);
		File ziel = new File(dest);
		if (ziel.exists())
			ziel.delete();
		boolean bla = quelle.renameTo(ziel);
		if (!bla) {
			System.err.println("Error: File cannot be removed");
		}
		
	}
	
	public static void copy(InputStream in, OutputStream out) throws IOException {
		byte[] buffer = new byte[1024];
		int i = 0;
		while ((i = in.read(buffer)) != -1) {
			out.write(buffer, 0, i);
		}
	}
	
	public static boolean copyFile(String src, String dest) {
		FileInputStream fis = null;
		FileOutputStream fos = null;
		
		try {
			fis = new FileInputStream(src);
			fos = new FileOutputStream(dest);
			
			copy(fis, fos);
		} catch (IOException e) {
			ErrorMsg.addErrorMessage(e);
			return false;
		} finally {
			if (fis != null)
				try {
					fis.close();
				} catch (IOException e) {
					ErrorMsg.addErrorMessage(e);
				}
			if (fos != null)
				try {
					fos.close();
				} catch (IOException e) {
					ErrorMsg.addErrorMessage(e);
				}
		}
		return true;
	}
	
	public static ExecutionParameter prepareExecution(Graph graph, boolean filesAndArray) {
		
		File sbmlf = new File(SBMLfile.getSBMLFilenameFromGraph(graph));
		File cmatrixf = new File(CreateCmatrix.getMatrixFilenameFromGraph(graph));
		boolean askUserForSBML = true, askUserForCMatrix = true;
		
		if (sbmlf.canRead() && cmatrixf.canRead()) {
			Object[] res = MyInputHelper.getInput(
					"<html>The following SBML- and C-Matrix-file were detected.<br>" +
							"Please specify, if you want to use them for calculation,<br>" +
							"otherwise you have to select them manually.<br>&nbsp;",
							"Select Configuration Files", new Object[] {
									"SBML:  " + sbmlf.getName(), true,
									"Matrix: " + cmatrixf.getName(), true,
			});
			if (res == null)
				return null;
			if ((Boolean) res[0])
				askUserForSBML = false;
			if ((Boolean) res[1])
				askUserForCMatrix = false;
			
			if (askUserForSBML)
				sbmlf = CreateSBMLFile.askUserForSBMLFile(sbmlf);
			
			if (askUserForCMatrix)
				cmatrixf = CreateCmatrix.askUserForCMatrixFile(cmatrixf);
		}
		
		if (!sbmlf.canRead()) {
			if (isSimulated(graph))
				checkForAlreadySimulated("SBML", graph);
			else {
				Object[] options = { "Reconstruct", "Choose File" };
				int n = JOptionPane.showOptionDialog(MainFrame.getInstance(),
										"<html>No SBML-file could be found here:<br>" +
												SBMLfile.getSBMLFilenameFromGraph(graph) + "<br>" +
												"You may reconstruct it or choose another file from the file system.",
										"Warning",
										JOptionPane.YES_NO_OPTION,
										JOptionPane.QUESTION_MESSAGE,
										null,
										options,
										options[1]);
				if (n == -1)
					return null;
				if (n == 0) {
					GravistoService.getInstance().runAlgorithm(new ModelReconstructionSelection(), null);
					return null;
				}
				if (n == 1)
					sbmlf = CreateSBMLFile.askUserForSBMLFile(sbmlf);
			}
		}
		if (!cmatrixf.canRead()) {
			if (isSimulated(graph))
				checkForAlreadySimulated("C-Matrix", graph);
			else {
				Object[] options = { "Reconstruct", "Choose File" };
				int n = JOptionPane.showOptionDialog(MainFrame.getInstance(),
										"<html>No C-Matrix-file could be found here:<br>" +
												CreateCmatrix.getMatrixFilenameFromGraph(graph) + "<br>" +
												"You may reconstruct it or choose another file from the file system.",
										"Warning",
										JOptionPane.YES_NO_OPTION,
										JOptionPane.QUESTION_MESSAGE,
										null,
										options,
										options[1]);
				if (n == -1)
					return null;
				if (n == 0) {
					GravistoService.getInstance().runAlgorithm(new ModelReconstructionSelection(), null);
					return null;
				}
				if (n == 1)
					cmatrixf = CreateCmatrix.askUserForCMatrixFile(cmatrixf);
			}
		}
		return prepareExecution(graph, filesAndArray, sbmlf, cmatrixf);
	}
	
	private static boolean isSimulated(Graph graph) {
		String gname = graph.getName().toLowerCase();
		String end = FileSaveAction.getFileExt(gname);
		if (end != null && end.length() > 0)
			gname = StringManipulationTools.stringReplace(gname, end, "");
		return gname.endsWith(FBAHelper.GRAPH_SIMULATED);
	}
	
	public static ExecutionParameter prepareExecution(Graph graph, boolean filesAndArray, File fileSBML, File cmatrix) {
		if (fileSBML == null || cmatrix == null)
			return null;
		
		ExecutionParameter par = new ExecutionParameter();
		
		par.filenameSBML = fileSBML.getName();
		
		String absPath = fileSBML.getAbsolutePath();
		String filename = fileSBML.getName();
		int index = absPath.indexOf(filename);
		par.path = absPath.substring(0, index);
		copyFile(par.path + par.filenameSBML, FBASimVizAddon.pathExeAddons + par.filenameSBML);
		
		String[][] rctNameArray = null;
		par.additionalConstraints = FBAHelper.checkConstraints(graph);
		
		// Erzeugung des rctNameArray
		Cmatrix matrix = new Cmatrix();
		try {
			if (par.additionalConstraints == true) {
				if (filesAndArray)
					rctNameArray = matrix.getFilesAndArray(cmatrix, FBASimVizAddon.pathExeAddons);
				else
					rctNameArray = matrix.getArray(cmatrix, FBASimVizAddon.pathExeAddons);
			} else {
				matrix.getFiles(cmatrix, FBASimVizAddon.pathExeAddons);
			}
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
		}
		
		if (par.additionalConstraints == true) {
			// Erzeugung des ub. & lb.txt-files
			Constraints cons = new Constraints();
			try {
				cons.generateUbLbFiles(graph, FBASimVizAddon.pathExeAddons, rctNameArray);
			} catch (IOException e1) {
				ErrorMsg.addErrorMessage(e1);
			}
		}
		return par;
	}
	
	public static void runCalculation(final BackgroundTaskStatusProviderSupportingExternalCall status, String name, final Runnable runnable) {
		// some background task...
		
		final int maxRun = 1;
		
		BackgroundTaskHelper.issueSimpleTask(name, "",
						new Runnable() {
							public void run() {
								
								if (!status.wantsToStop()) {
									status.setCurrentStatusText1("Start Simulation...");
									for (int run = 1; run <= maxRun; run++) {
										status.setCurrentStatusText2("Run simulation ");
										runnable.run();
										if (status.wantsToStop())
											break;
									}
									if (status.wantsToStop()) {
										// delete temporary files...
										// clean up
										status.setCurrentStatusText1("Processing aborted!");
									} else {
										// process simulation results...
										// ...
										status.setCurrentStatusText1("Processing finished");
									}
								}
							}
						}, null, status);
		
	}
	
	public static void checkForAlreadySimulated(String fname, Graph g) {
		
		Object[] options = { "Stop", "Open original graph" };
		int n = JOptionPane.showOptionDialog(MainFrame.getInstance(),
								"<html>No " + fname + "-file available, but the graph name ends with<br>" +
										"\"" + FBAHelper.GRAPH_SIMULATED + "\", which indicates, that a simulation was performed<br>" +
										"on the original graph and the actual new (renamed) graph was created<br>" +
										"in order to prevent overwriting original graphs. You still may continue<br>" +
										"to simulate the actual graph by reconstructing the model using menu entry<br>" +
										"\"FBA\"&rarr;\"Model Reconstruction\" beforehand. Alternatively you can<br>" +
										"open the original graph.",
								"Warning",
								JOptionPane.YES_NO_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,
								options,
								options[1]);
		
		if (n == 1) {
			File file = new File(StringManipulationTools.stringReplace(g.getName(true), FBAHelper.GRAPH_SIMULATED, ""));
			MainFrame.getInstance().loadGraph(file);
		}
		
	}
	
	public static Graph getGraph() {
		EditorSession es = MainFrame.getInstance().getActiveEditorSession();
		if (es != null && es.getGraph() != null)
			return es.getGraph();
		else {
			MainFrame.showMessageDialog("All graphs closed, please also close this dialog.", "Error");
			return new AdjListGraph();
		}
	}
	
	public static String getSeriesNameForComparison(ConditionInterface sd) {
		String sdName = sd.getName();
		return sdName.substring(sdName.indexOf(": ") + ": ".length());
	}
	
	public static int getNbrOfHeads(Edge ed) {
		int i = 0;
		String head1 = AttributeHelper.getArrowhead(ed);
		if (!(head1 == null || head1.length() <= 0))
			i++;
		String head2 = AttributeHelper.getArrowtail(ed);
		if (!(head2 == null || head2.length() <= 0))
			i++;
		return i;
	}
	
	public static boolean isVisuallyInvertedEdge(Edge ed) {
		String head = AttributeHelper.getArrowhead(ed);
		String tail = AttributeHelper.getArrowtail(ed);
		return (head == null || head.length() <= 0) && tail != null && tail.length() > 0;
	}
	
	public static void renameGraphAfterFluxSimulation(Graph graph) {
		String gname = graph.getName(true);
		String shortname = graph.getName(false);
		String end = FileSaveAction.getFileExt(gname);
		if (end != null && end.length() > 0) {
			String name = shortname.substring(0, shortname.indexOf("."));
			name = StringManipulationTools.stringReplace(name, GRAPH_SIMULATED, "");
			String newname = StringManipulationTools.stringReplace(gname, shortname, "") + name + GRAPH_SIMULATED + end;
			graph.setName(newname);
			MainFrame.getInstance().sessionChanged(MainFrame.getInstance().getActiveEditorSession());
		} else
			ErrorMsg.addErrorMessage("Could not rename the graph. Please save this graph in another file.");
		
	}
	
}
