package de.ipk_gatersleben.ag_pbi.fluxbalance.unittests;

import java.io.File;

import org.ErrorMsg;
import org.ReleaseInfo;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.selection.Selection;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.Experiment;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.calculation.StartCalculationAlgorithmMultiFinalNEW;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateCmatrix;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.SBMLfile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.ExecutionParameter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBASimVizAddon;

public class TestCalculation {
	
	private final String[] graphnames = new String[] {
			"sample_network.gml", "photorespiration.gml", "Photophosphorylation.gml", "Calvin_cycle.gml", "C4_core.gml",
			"C4 and Calvin_backup.gml", "C4 and Calvin and Photorespiration_backup.gml", "C4 Photosynthesis.gml"
	};
	
	@Test
	public void performFBA() throws Exception {
		FBASimVizAddon.pathExeAddons = ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "FBASimVis");
		FBASimVizAddon.copyFilesToFBADir();
		
		int errcnt = 0;
		for (Graph g : TestReconstruction.getGraphs()) {
			
			StartCalculationAlgorithmMultiFinalNEW alg = new StartCalculationAlgorithmMultiFinalNEW(true);
			alg.attach(g, new Selection("none"));
			alg.check();
			setFBAParameters(alg, g);
			
			ExecutionParameter par = FBAHelper.prepareExecution(g, true,
					new File(SBMLfile.getSBMLFilenameFromGraph(g)), new File(CreateCmatrix.getMatrixFilenameFromGraph(g)));
			
			alg.doExecute(g, par);
			waitForAlgorithm(alg);
			
			checkFBAMapping(g);
			
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	private void checkFBAMapping(Graph g) {
		Experiment e = new Experiment();
		
		if (g.getName().equals(graphnames[3]) || g.getName().equals(graphnames[4]))
			return; // TODO: check, why there is a concurrentmodification exception during merging of mappingdata of this graphs
			
		for (GraphElement ge : g.getGraphElements()) {
			e.addAndMerge(new GraphElementHelper(ge).getDataMappings());
		}
		
		if (g.getName().equals(graphnames[0]))
			Assert.assertEquals(17, e.getAllMeasurements().size());
		else
			if (g.getName().equals(graphnames[1]))
				Assert.assertEquals(69, e.getAllMeasurements().size());
			else
				if (g.getName().equals(graphnames[2]))
					Assert.assertEquals(26, e.getAllMeasurements().size());
				else
					if (g.getName().equals(graphnames[3]))
						Assert.assertEquals(e.getAllMeasurements().size(), 73);
					else
						if (g.getName().equals(graphnames[4]))
							Assert.assertEquals(e.getAllMeasurements().size(), 45);
						else
							if (g.getName().equals(graphnames[5]))
								Assert.assertEquals(118, e.getAllMeasurements().size());
							else
								if (g.getName().equals(graphnames[6]))
									Assert.assertEquals(191, e.getAllMeasurements().size());
								else
									if (g.getName().equals(graphnames[7]))
										Assert.assertEquals(244, e.getAllMeasurements().size());
		
	}
	
	private void setFBAParameters(AbstractStartCalculationAlgorithm alg, Graph g) {
		Parameter[] params = alg.getParameters();
		if (g.getName().equals(graphnames[0]))
			params[0].setValue("bz");
		else
			if (g.getName().equals(graphnames[1]))
				params[0].setValue("R019");
			else
				if (g.getName().equals(graphnames[2]))
					params[0].setValue("R008");
				else
					if (g.getName().equals(graphnames[3]))
						params[0].setValue("R019");
					else
						if (g.getName().equals(graphnames[4]))
							params[0].setValue("R009");
						else
							if (g.getName().equals(graphnames[5]))
								params[0].setValue("R119");
							else
								if (g.getName().equals(graphnames[6]))
									params[0].setValue("R119");
								else
									if (g.getName().equals(graphnames[7]))
										params[0].setValue("R119");
									else
										Assert.fail("Unknown Graph!");
		alg.setParameters(params);
	}
	
	@Ignore
	@Test
	public void performKnockout() throws Exception {
		FBASimVizAddon.pathExeAddons = ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "FBASimVis");
		FBASimVizAddon.copyFilesToFBADir();
		
		int errcnt = 0;
		for (Graph g : TestReconstruction.getGraphs()) {
			
			StartCalculationAlgorithmMultiFinalNEW alg = new StartCalculationAlgorithmMultiFinalNEW(true);
			alg.attach(g, new Selection("none"));
			alg.check();
			// setFBAParameters(alg, g);
			
			ExecutionParameter par = FBAHelper.prepareExecution(g, true,
					new File(SBMLfile.getSBMLFilenameFromGraph(g)), new File(CreateCmatrix.getMatrixFilenameFromGraph(g)));
			
			alg.doExecute(g, par);
			waitForAlgorithm(alg);
			
			// checkFBAMapping(g);
			
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	private void waitForAlgorithm(StartCalculationAlgorithmMultiFinalNEW alg) {
		System.out.print("waiting for simulation");
		while (BackgroundTaskHelper.isTaskWithGivenReferenceRunning(alg.getName())) {
			try {
				Thread.sleep(500);
				System.out.print(".");
			} catch (InterruptedException ie) {
				ErrorMsg.addErrorMessage(ie);
			}
		}
		System.out.println("finished");
		alg.reset();
	}
	
}
