package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JComponent;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.selection.Selection;

import de.ipk_gatersleben.ag_nw.graffiti.MyInputHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AnimatedFluxVisualizationDeletion extends AbstractAlgorithm {
	
	public void execute() {
		try {
			// min, max values berechnen von gesamten Ergebnissen
			double minValue = Double.MAX_VALUE;
			double maxValue = Double.MIN_VALUE;
			
			for (GraphElement ge : graph.getGraphElements()) {
				GraphElementHelper geh = new GraphElementHelper(ge);
				for (SubstanceInterface md : geh.getDataMappings()) {
					for (ConditionInterface sd : md) {
						for (SampleInterface sampleData : sd) {
							double avg = Math.abs(sampleData.calcMean());
							if (!Double.isNaN(avg)) {
								// if (avg<minValue)
								// minValue = avg;
								if (avg > maxValue)
									maxValue = avg;
							}
						}
					}
				}
			}
			minValue = 0;
			
			// Eva: 15.11.
			// VersionA: Visualization durch getrennte betaetigung des Panels (fruehere Version)
			// Collection<String> seriesList1 = StartSimulationAlgorithm.getSeriesNames(getSelectedOrAllGraphElements());
			
			Collection<String> seriesList1 = StartSimulationAlgorithm.getSeriesNames(graph.getGraphElements());
			
			// arrayList der commandbuttons
			ArrayList<JComponent> buttons = new ArrayList<JComponent>();
			
			// jedem commandbutton wird ein Parameter (series) zugewiesen
			
			for (String series : seriesList1) {
				
				JComponent commandButton = getCommandButton(series, seriesList1, minValue, maxValue);
				buttons.add(commandButton);
				
			}
			JComponent buttonPanel = null;
			switch (buttons.size()) {
				case 0:
					MainFrame.showMessageDialog("<html>There are no flux distributions in the active graph!", "Cannot Visualise Fluxes");
					break;
				case 1:
					((JButton) buttons.get(0)).doClick();
					
					buttonPanel = TableLayout.getMultiSplit(buttons);
					MyInputHelper.getInput("[nonmodal,Close]Click button to visualise corresponding flux distribution!", "Flux Data Visualisation",
										new Object[] { "Knock-out reactions ", buttonPanel
					});
				default:
					buttonPanel = TableLayout.getMultiSplit(buttons);
					MyInputHelper.getInput("[nonmodal,Close]Click any button to visualise corresponding flux distribution!", "Flux Data Visualisation",
										new Object[] { "Knock-out reactions ", buttonPanel
					});
					break;
			}
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
	}
	
	private JComponent getCommandButton(final String series, final Collection<String> allSeries, final double minVal, final double maxVal) {
		
		JButton button = new JButton(series);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				StartSimulationAlgorithm sa = new StartSimulationAlgorithm(minVal, maxVal);
				sa.attach(FBAHelper.getGraph(), new Selection("none"));
				sa.setParameters(new Parameter[] {
									new ObjectListParameter(series, "Simulationsparameter", "", allSeries)
				});
				
				try {
					sa.check();
					sa.execute();
					
					for (GraphElement ge : FBAHelper.getGraph().getGraphElements()) {
						String name = AttributeHelper.getSBMLid(ge);
						if (name.equals(series)) {
							AttributeHelper.setOutlineColor(ge, Color.ORANGE);
							AttributeHelper.setLabelColor(-1, (Node) ge, Color.ORANGE);
							AttributeHelper.setBorderWidth((Node) ge, 2);
							break;
						}
					}
				} catch (PreconditionException e1) {
					ErrorMsg.addErrorMessage(e1);
				}
			}
		});
		return button;
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	public String getName() {
		return "<html><font color=\"#04B4AE\">Knock-out Analysis (specified)</font>";
	}
}
