package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import java.awt.Color;
import java.util.Collection;

import org.AlignmentSetting;
import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.NodeTools;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AnimatedFluxVisualizationDeletionComplete extends AbstractAlgorithm {
	
	private double avg;
	
	public void execute() {
		try {
			String checkOptSol = "optSol";
			
			// Erstellung der minFlux-/maxFlux-Value Diagramms
			checkOptSol = createDiagram(graph, graph.getNodes());
			GraphHelper.issueCompleteRedrawForGraph(graph);
			if (checkOptSol.equals("NoOptSol")) {
				MainFrame.showMessageDialog("NOS: No optimal solution found!", "Information");
			}
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
	}
	
	public String createDiagram(Graph g, Collection<Node> nodes) {
		
		String checkOptSol = "optSol";
		int index = 0;
		
		AttributeHelper.setAttribute(g, "", "node_plotAxisFontSize", 32);
		AttributeHelper.setAttribute(g, "", "node_showCategoryAxis", false);
		AttributeHelper.setAttribute(g, "", "node_showGridCategory", true);
		AttributeHelper.setAttribute(g, "", "node_showGridRange", true);
		AttributeHelper.setAttribute(g, "", "node_showRangeAxis", true);
		AttributeHelper.setAttribute(g, "", "node_gridWidth", 1.0);
		AttributeHelper.setAttribute(g, "", "node_axisWidth", 3.0);
		AttributeHelper.setAttribute(g, "", "node_outlineBorderWidth", 1.0);
		AttributeHelper.setAttribute(g, "", "node_chartShapeSize", 6.0);
		
		for (Node n : nodes) {
			
			if (FBAHelper.isReaction(n)) {
				
				AttributeHelper.setSize(n, 50, 50);
				AttributeHelper.setLabelAlignment(-1, n, AlignmentSetting.BELOW);
				NodeTools.setNodeComponentType(n, GraffitiCharts.BAR_FLAT.getName());
				
				String rangeAxisLabel = "% optimal OF-value";
				AttributeHelper.setAttribute(n, "charting", "rangeAxis", rangeAxisLabel);
				AttributeHelper.setAttribute(n, "charting", "useCustomRange", true);
				AttributeHelper.setAttribute(n, "charting", "minRange", 0.0);
				AttributeHelper.setAttribute(n, "charting", "maxRange", 100.0);
				
				// double avg = 0.0;
				// GraphElementHelper geh = new GraphElementHelper(n);
				// for (SubstanceInterface md : geh.getDataMappings()) {
				// for (ConditionInterface sd : md) {
				// for (SampleInterface sampleData : sd) {
				// avg = Math.abs(sampleData.calcMean());
				//
				// }
				// }
				// }
				// String value = ("Percentage of optimal OF-value (deleted/undeleted scenario): " + String.valueOf(avg) + "%.");
			}
		}
		
		// Einfärben der Knoten mit Flux variabilitaet
		// for (GraphElement ge : getSelectedOrAllGraphElements()) {
		for (GraphElement ge : graph.getNodes()) {
			GraphElementHelper geh = new GraphElementHelper(ge);
			for (SubstanceInterface md : geh.getDataMappings()) {
				for (ConditionInterface sd : md) {
					for (SampleInterface sampleData : sd) {
						avg = sampleData.calcMean();
						
					}
				}
				if (avg == 0) {
					AttributeHelper.setOutlineColor(ge, Color.ORANGE);
					AttributeHelper.setLabelColor(-1, (Node) ge, Color.ORANGE);
					AttributeHelper.setBorderWidth(ge, 2);
				} else {
					AttributeHelper.setOutlineColor(ge, Color.BLACK);
					AttributeHelper.setLabelColor(-1, (Node) ge, Color.BLACK);
					AttributeHelper.setBorderWidth(ge, 1);
				}
				
				if (Double.isNaN(avg) || Double.isInfinite(avg)) {
					System.out.println("!!!!!!NOS");
					++index;
					checkOptSol = "NoOptSol";
					System.out.println("no opt Solution: " + index);
					AttributeHelper.setLabel(3, (Node) ge, "NOS", "Arial", null);
					AttributeHelper.setLabelAlignment(3, (Node) ge, AlignmentSetting.CENTERED);
					AttributeHelper.setLabelColor(3, (Node) ge, Color.RED);
					NodeTools.setNodeComponentType(ge, GraffitiCharts.HIDDEN.getName());
				} else {
					AttributeHelper.setLabel(3, (Node) ge, "", "Arial", null);
					NodeTools.setNodeComponentType(ge, GraffitiCharts.BAR.getName());
				}
			}
		}
		if (index > 0) {
			checkOptSol = "NoOptSol";
		}
		return checkOptSol;
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	public String getName() {
		// return "Knock-out Analysis (complete)";
		return "<html><font color=\"#04B45F\">Knock-out Analysis (complete)</font>";
		
	}
}
