package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import java.awt.Color;
import java.util.Collection;

import org.AlignmentSetting;
import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.NodeTools;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AnimatedFluxVisualizationFluxVariability extends AbstractAlgorithm {
	
	private double min;
	private double max;
	private double minValue;
	private double maxValue;
	
	public void execute() {
		try {
			String checkOptSol = "optSol";
			
			// Erstellung der minFlux-/maxFlux-Value Diagramms
			checkOptSol = createDiagram(graph, graph.getNodes());
			GraphHelper.issueCompleteRedrawForGraph(graph);
			if (checkOptSol.equals("NoOptSol")) {
				MainFrame.showMessageDialog("NOS: No optimal solution found!", "Information");
			}
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
	}
	
	public String createDiagram(Graph g, Collection<Node> nodes) {
		
		String checkOptSol = "optSol";
		int index = 0;
		
		AttributeHelper.setAttribute(g, "", "node_plotAxisFontSize", 33);
		AttributeHelper.setAttribute(g, "", "node_showCategoryAxis", false);
		AttributeHelper.setAttribute(g, "", "node_showGridCategory", true);
		AttributeHelper.setAttribute(g, "", "node_showGridRange", true);
		AttributeHelper.setAttribute(g, "", "node_showRangeAxis", true);
		AttributeHelper.setAttribute(g, "", "node_gridWidth", 1.0);// default
		AttributeHelper.setAttribute(g, "", "node_axisWidth", 3.0);
		AttributeHelper.setAttribute(g, "", "node_outlineBorderWidth", 5.0);
		AttributeHelper.setAttribute(g, "", "node_chartShapeSize", 6.0);
		
		for (Node n : nodes) {
			if (FBAHelper.isReaction(n)) {
				AttributeHelper.setSize(n, 50, 50);
				AttributeHelper.setLabelAlignment(-1, n, AlignmentSetting.BELOW);
				NodeTools.setNodeComponentType(n, GraffitiCharts.BAR_FLAT.getName());
				String rangeAxisLabel = "C-flux";
				AttributeHelper.setAttribute(n, "charting", "rangeAxis", rangeAxisLabel);
				
				GraphElementHelper geh = new GraphElementHelper(n);
				for (SubstanceInterface md : geh.getDataMappings()) {
					for (ConditionInterface sd : md) {
						for (SampleInterface sampleData : sd) {
							String sdName = FBAHelper.getSeriesNameForComparison(sd);
							if (sdName.equals("min")) {
								minValue = sampleData.calcMean();
								if (minValue < 0.0001) {
									minValue = 0;
								}
							} else
								if (sdName.equals("max")) {
									maxValue = sampleData.calcMean();
									if (maxValue < 0.0001) {
										maxValue = 0;
									}
								}
						}
					}
				}
				String value = ("Minimal C-flux value: " + String.valueOf(minValue) + ", maximal C-flux value: " + String.valueOf(maxValue) + ".");
				AttributeHelper.setToolTipText(n, value);
			}
		}
		
		// Einfärben der Knoten mit Flux variabilitaet
		for (Node nd : getSelectedOrAllNodes()) {
			
			GraphElementHelper geh = new GraphElementHelper(nd);
			for (SubstanceInterface md : geh.getDataMappings()) {
				int i = 0;
				for (ConditionInterface sd : md) {
					for (SampleInterface sampleData : sd) {
						double avg = sampleData.calcMean();
						
						// Check ob no optimal Solution found
						if (Double.isNaN(avg) || Double.isInfinite(avg)) {
							++index;
							checkOptSol = "NoOptSol";
							AttributeHelper.setLabel(3, nd, "NOS", "Arial", null);
							AttributeHelper.setLabelAlignment(3, nd, AlignmentSetting.CENTERED);
							AttributeHelper.setLabelColor(3, nd, Color.RED);
							NodeTools.setNodeComponentType(nd, GraffitiCharts.HIDDEN.getName());
						} else {
							AttributeHelper.setLabel(3, nd, "", "Arial", null);
							NodeTools.setNodeComponentType(nd, GraffitiCharts.BAR_FLAT.getName());
						}
						
						if (avg < 0.0001) {
							avg = 0;
						}
						if (i == 0) {
							min = avg;
							i++;
						} else {
							max = avg;
							i++;
						}
					}
				}
				double sum = Math.abs(max - min);
				double result = Math.rint(sum * 100.0) / 100.0;
				if (result != 0) {
					AttributeHelper.setOutlineColor(nd, Color.ORANGE);
					AttributeHelper.setLabelColor(-1, nd, Color.ORANGE);
					AttributeHelper.setBorderWidth(nd, 2);
				}
			}
		}
		
		if (index > 0) {
			checkOptSol = "NoOptSol";
		}
		return checkOptSol;
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	public String getName() {
		// return "Flux Variability Analysis";
		return "<html><font color=\"#5F04B4\">Flux Variability Analysis</font>";
	}
}
