package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import java.awt.Color;
import java.util.Collection;

import org.AlignmentSetting;
import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.NodeTools;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AnimatedFluxVisualizationRobustnessComplete extends AbstractAlgorithm {
	
	public void execute() {
		try {
			String checkOptSol = "optSol";
			
			// Erstellung des OF-Rct Diagramms
			checkOptSol = createDiagram(graph, graph.getNodes());
			GraphHelper.issueCompleteRedrawForGraph(graph);
			if (checkOptSol.equals("NoOptSol")) {
				MainFrame.showMessageDialog("NOS: No optimal solution found!", "Information");
			}
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
	}
	
	public String createDiagram(Graph g, Collection<Node> nodes) {
		
		String checkOptSol = "optSol";
		int index = 0;
		
		// globale Attribute (direkt an Graph hängend) setzen:
		// AttributeHelper.setAttribute(attributable, path, attributeName, attributeValue)
		
		AttributeHelper.setAttribute(g, "", "node_plotAxisFontSize", 20);
		AttributeHelper.setAttribute(g, "", "node_showCategoryAxis", true);
		AttributeHelper.setAttribute(g, "", "node_showGridCategory", true);
		AttributeHelper.setAttribute(g, "", "node_showGridRange", true);
		AttributeHelper.setAttribute(g, "", "node_showRangeAxis", true);
		AttributeHelper.setAttribute(g, "", "node_gridWidth", 1.0);// default
		AttributeHelper.setAttribute(g, "", "node_plotAxisSteps", 2.0);
		AttributeHelper.setAttribute(g, "", "node_axisWidth", 3.0);
		AttributeHelper.setAttribute(g, "", "node_outlineBorderWidth", 5.0);
		AttributeHelper.setAttribute(g, "", "node_chartShapeSize", 6.0);
		
		for (Node n : nodes) {
			
			if (FBAHelper.isReaction(n)) {
				String label = AttributeHelper.getLabel(n, "");
				
				AttributeHelper.setOutlineColor(n, Color.BLUE);
				AttributeHelper.setLabelColor(-1, n, Color.BLUE);
				AttributeHelper.setBorderWidth(n, 1);
				AttributeHelper.setSize(n, 35, 35);
				AttributeHelper.setLabelAlignment(-1, n, AlignmentSetting.BELOW);
				NodeTools.setNodeComponentType(n, GraffitiCharts.LINE.getName());
				
				String rangeAxisLabel = "Objective function";
				String domainAxisLabel = "Parameter: " + label;
				
				AttributeHelper.setAttribute(n, "charting", "rangeAxis", rangeAxisLabel);
				AttributeHelper.setAttribute(n, "charting", "domainAxis", domainAxisLabel);
				
				AttributeHelper.setAttribute(n, "charting", "useCustomRange", true);
				AttributeHelper.setAttribute(n, "charting", "minRange", 0.0);
				AttributeHelper.setAttribute(n, "charting", "maxRange", 100.0);
			}
		}
		
		// Check Opt sol
		for (GraphElement ge : graph.getNodes()) {
			// for (GraphElement ge : getSelectedOrAllGraphElements()) {
			
			GraphElementHelper geh = new GraphElementHelper(ge);
			for (SubstanceInterface md : geh.getDataMappings()) {
				for (ConditionInterface sd : md) {
					for (SampleInterface sampleData : sd) {
						double avg = sampleData.calcMean();
						
						// Check ob no optimal Solution found
						if (Double.isNaN(avg) || Double.isInfinite(avg)) {
							++index;
							checkOptSol = "NoOptSol";
							AttributeHelper.setLabel(3, (Node) ge, "NOS", "Arial", null);
							AttributeHelper.setLabelAlignment(3, (Node) ge, AlignmentSetting.CENTERED);
							AttributeHelper.setLabelColor(3, (Node) ge, Color.RED);
							NodeTools.setNodeComponentType(ge, GraffitiCharts.HIDDEN.getName());
						} else {
							AttributeHelper.setLabel(3, (Node) ge, "", "Arial", null);
							NodeTools.setNodeComponentType(ge, GraffitiCharts.LINE.getName());
						}
					}
				}
			}
		}
		
		if (index > 0) {
			checkOptSol = "NoOptSol";
		}
		return checkOptSol;
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	public String getName() {
		// return "Robustness Analysis (complete)";
		return "<html><font color=\"#045FB4\">Robustness Analysis (complete)</font>";
	}
}
