package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import java.awt.Color;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.AlignmentSetting;
import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.selection.Selection;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.MyInputHelper;
import de.ipk_gatersleben.ag_nw.graffiti.NodeTools;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class AnimatedFluxVisualizationSliderRobustness extends AbstractAlgorithm {
	
	private final int varMin = 0;
	private final int varMax = 100;
	
	public void execute() {
		try {
			// min, max values berechnen von gesamten Ergebnissen
			double minValue = Double.MAX_VALUE;
			double maxValue = Double.MIN_VALUE;
			
			for (GraphElement ge : graph.getGraphElements()) {
				GraphElementHelper geh = new GraphElementHelper(ge);
				for (SubstanceInterface md : geh.getDataMappings()) {
					for (ConditionInterface sd : md) {
						for (SampleInterface sampleData : sd) {
							double avg = Math.abs(sampleData.calcMean());
							if (!Double.isNaN(avg)) {
								// if (avg<minValue)
								// minValue = avg;
								if (avg > maxValue)
									maxValue = avg;
							}
						}
					}
				}
			}
			minValue = 0;
			
			// Collection<String> seriesList = StartSimulationAlgorithm.getSeriesNames(getSelectedOrAllGraphElements());
			Collection<String> seriesList = StartSimulationAlgorithm.getSeriesNames(graph.getGraphElements());
			// Anzahl vorhandener PArameterwerte = Anzahl visualisierbarer Flux distributions
			int noTicks = seriesList.size();
			
			// Create the label table
			Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
			int[] valArray = new int[noTicks];
			
			int i = 0;
			
			// LAbel Parameter setzen
			Iterator<String> it = seriesList.iterator();
			String vpRctName = it.next();
			
			int index = vpRctName.lastIndexOf("<*>", vpRctName.length());
			int indexOF = vpRctName.lastIndexOf("_", vpRctName.length());
			
			while (index == -1) {
				vpRctName = it.next();
				index = vpRctName.lastIndexOf("<*>", vpRctName.length());
				indexOF = vpRctName.lastIndexOf("_", vpRctName.length());
			}
			String parameter = vpRctName.substring(0, indexOF);
			String of = vpRctName.substring(indexOF + 1, index);
			for (String series : seriesList) {
				int index2 = series.lastIndexOf("<*>", vpRctName.length());
				if (index2 == -1) {
				} else {
					String parameterValue = series.substring(index + 4, series.length());
					Integer val = new Integer(parameterValue).intValue();
					String parameterValue2 = Integer.toString(val);
					labelTable.put(new Integer(val), new JLabel(parameterValue2));
					valArray[i++] = val;
				}
			}
			
			JSlider slider = getSlider(seriesList, labelTable, varMin, varMax, noTicks, minValue, maxValue);
			
			// Erstellung des OF-Rct Diagramms
			createDiagram(graph, graph.getNodes(), parameter, of);
			GraphHelper.issueCompleteRedrawForGraph(graph);
			MyInputHelper.getInput("[nonmodal,Close]Constrain flux to visualise corresponding flux distribution", "Flux Visualisation",
								new Object[] {
													parameter, slider
			});
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		}
	}
	
	public void createDiagram(Graph g, Collection<Node> nodes, String parameter, String of) {
		
		AttributeHelper.setAttribute(g, "", "node_plotAxisFontSize", 20);
		AttributeHelper.setAttribute(g, "", "node_showCategoryAxis", true);
		AttributeHelper.setAttribute(g, "", "node_showGridCategory", true);
		AttributeHelper.setAttribute(g, "", "node_showGridRange", true);
		AttributeHelper.setAttribute(g, "", "node_showRangeAxis", true);
		AttributeHelper.setAttribute(g, "", "node_gridWidth", 1.0);// default
		AttributeHelper.setAttribute(g, "", "node_plotAxisSteps", 4.0);
		AttributeHelper.setAttribute(g, "", "node_axisWidth", 3.0);
		AttributeHelper.setAttribute(g, "", "node_outlineBorderWidth", 5.0);
		AttributeHelper.setAttribute(g, "", "node_chartShapeSize", 6.0);
		
		for (Node n : nodes) {
			String label = AttributeHelper.getLabel(n, "");
			if (label.equals(parameter)) {
				AttributeHelper.setOutlineColor(n, Color.BLUE);
				AttributeHelper.setLabelColor(-1, n, Color.BLUE);
				AttributeHelper.setBorderWidth(n, 2);
			}
			if (label.equals(of)) {
				AttributeHelper.setOutlineColor(n, Color.CYAN);
				AttributeHelper.setLabelColor(-1, n, Color.CYAN);
				AttributeHelper.setBorderWidth(n, 2);
				AttributeHelper.setSize(n, 100, 100);
				AttributeHelper.setLabelAlignment(-1, n, AlignmentSetting.BELOW);
				NodeTools.setNodeComponentType(n, GraffitiCharts.LINE.getName());
				String rangeAxisLabel = "Objective function: " + of;
				String domainAxisLabel = "Parameter: " + parameter;
				AttributeHelper.setAttribute(n, "charting", "rangeAxis", rangeAxisLabel);
				AttributeHelper.setAttribute(n, "charting", "domainAxis", domainAxisLabel);
				AttributeHelper.setAttribute(n, "charting", "useCustomRange", true);
				AttributeHelper.setAttribute(n, "charting", "minRange", 0.0);
				AttributeHelper.setAttribute(n, "charting", "maxRange", 100.0);
			}
		}
	}
	
	private JSlider getSlider(final Collection<String> allSeries, final Hashtable<Integer, JLabel> labelTable, final int varMin, final int varMax,
			final int noTicks,
						final double minVal, final double maxVal) {
		
		JSlider slider = new JSlider();
		slider.setMinimum(varMin);
		slider.setMaximum(varMax);
		slider.setMinorTickSpacing(5);
		slider.setMajorTickSpacing(10);
		slider.setPaintLabels(true);
		slider.setPaintTicks(true);
		slider.setLabelTable(labelTable);
		slider.setValue(varMin);
		slider.setSnapToTicks(true);
		
		slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent ce) {
				JSlider slider = (JSlider) ce.getSource();
				
				if (!slider.getValueIsAdjusting()) {
					int value = slider.getValue();
					String str = Integer.toString(value);
					String out = ("P" + str);
					StartSimulationAlgorithm sa = new StartSimulationAlgorithm(minVal, maxVal);
					sa.attach(FBAHelper.getGraph(), new Selection("none"));
					sa.setParameters(new Parameter[] {
										new ObjectListParameter(out, "Simulationsparameter", "", allSeries)
					});
					try {
						sa.check();
						sa.execute();
					} catch (PreconditionException e1) {
						ErrorMsg.addErrorMessage(e1);
					}
				}
			}
		});
		return slider;
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	public String getName() {
		// return "Robustness Analysis (specified)";
		return "<html><font color=\"#0404B4\">Robustness Analysis (specified)</font>";
	}
}
